extends CharacterBody2D

@onready var TileMapLayerFloor: TileMapLayer = %floor
@onready var TileMapLayerWalls: TileMapLayer = %walls
@onready var map: Node2D = get_parent()

const speed = 500

var astargrid = AStarGrid2D.new()
var rally_points: Array[Area2D] = []
var current_path: PackedVector2Array = []
var target_point_index = 0
var current_path_line: Line2D = null

func _ready():
	initialize_astargrid2d()
	get_rally_targets()
	set_next_target()

func _process(_delta):
	if Input.is_mouse_button_pressed(MOUSE_BUTTON_MIDDLE):
		print_positions()
	if !current_path.is_empty():
		var next_point = current_path[0]
		var direction = (next_point - global_position).normalized()
		
		velocity = direction * speed
		move_and_slide()

		if global_position.distance_to(next_point) < 5:
			current_path.remove_at(0)
			if current_path.is_empty():
				set_next_target()


func initialize_astargrid2d():
	astargrid.region = TileMapLayerFloor.get_used_rect()
	astargrid.cell_size = TileMapLayerFloor.tile_set.tile_size
	astargrid.jumping_enabled = true
	astargrid.diagonal_mode = AStarGrid2D.DIAGONAL_MODE_ONLY_IF_NO_OBSTACLES
	astargrid.update()

	for tile in TileMapLayerWalls.get_used_cells():
		astargrid.set_point_solid(tile)

	astargrid.update()

func get_rally_targets():
	var rally_nodes = get_tree().get_nodes_in_group("RallyPoints")
	for node in rally_nodes:
		if node is Area2D:
			rally_points.append(node)
	
	rally_points.sort()

func set_next_target():
	if target_point_index < rally_points.size():
		var target = rally_points[target_point_index]
		var start_point = TileMapLayerFloor.local_to_map(global_position)
		var end_point = TileMapLayerFloor.local_to_map(target.global_position)
		var path = astargrid.get_point_path(start_point, end_point)

		if path.is_empty():
			print("No path found to target ", target_point_index + 1)
			target_point_index += 1
			set_next_target()
		else:
			current_path.clear()
			current_path = path
			target_point_index += 1
			print_path(path)
	else:
		print("All targets reached")
		queue_free()


func print_path(path: PackedVector2Array) -> void:
	if current_path_line:
		current_path_line.queue_free()
	
	var world_path: PackedVector2Array = []
	for point:Vector2i in path:
		world_path.append(point + Vector2i(astargrid.cell_size) / 2)
	
	current_path_line = Line2D.new()
	map.add_child(current_path_line)
	current_path_line.width = 2
	current_path_line.default_color = Color.LIGHT_GREEN
	current_path_line.begin_cap_mode = Line2D.LINE_CAP_ROUND
	current_path_line.end_cap_mode = Line2D.LINE_CAP_ROUND
	current_path_line.joint_mode = Line2D.LINE_JOINT_ROUND
	current_path_line.points = world_path

#-------------DEBUGGING-------------

func print_positions() -> void:
	var global_pos = get_global_mouse_position()
	var tilemap_pos_floor = TileMapLayerFloor.local_to_map(global_pos)
	var in_bounds = astargrid.is_in_bounds(tilemap_pos_floor.x, tilemap_pos_floor.y)
	var is_solid = astargrid.is_point_solid(tilemap_pos_floor)
	print("get_tree:")
	get_tree().root.print_tree()
	print("Global Position: ", global_pos)
	print("Map Position Floor: ", tilemap_pos_floor)
	print("in_bounds: ", in_bounds)
	print("is_solid: " , is_solid)
