extends CharacterBody2D

const speed = 200

@onready var nav_agent: NavigationAgent2D = $NavigationAgent2D

var rally_points: Array[Area2D] = []
var current_target_index: int = 0

func _ready():
	get_rally_targets()
	calculate_path()

func _physics_process(delta: float) -> void:
	move()

func get_rally_targets():
	var rally_nodes = get_tree().get_nodes_in_group("RallyPoints")
	for node in rally_nodes:
		if node is Area2D:
			rally_points.append(node)

	rally_points.sort()

func calculate_path() -> void:
	if current_target_index < rally_points.size():
		nav_agent.target_position = rally_points[current_target_index].global_position
	else:
		print(self, " FINAL TARGET REACHED")
		self.queue_free()

func is_rally_point_reached() -> bool:
	if current_target_index < rally_points.size():
		var current_rally_point = rally_points[current_target_index]
		return current_rally_point.overlaps_body(self)
	return false

func move():
	if is_rally_point_reached():
		print(self, " RALLY POINT REACHED")
		current_target_index += 1
		calculate_path()
	else:
		var dir = to_local(nav_agent.get_next_path_position()).normalized()
		velocity = dir * speed
		move_and_slide()
	
